/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <msp430.h>
#include <driverlib.h>

#include "peripherals.h"

//#define HOLD									// MANUAL | ENABLE | TEST | HOLD

#define NRADCCONVMAX 400						// # DMA transfers;

__attribute__((section(".appdata"))) uint16_t actualData[NRADCCONVMAX];
__attribute__((section(".appdata"))) uint16_t avrgData[NRADCCONVMAX];
__attribute__((section(".appdata"))) uint16_t saveData[NRADCCONVMAX];

__attribute__((section(".appdata"))) uint16_t maxCurrent = 0;
__attribute__((section(".appdata"))) uint16_t maxCount = 0;
__attribute__((section(".appdata"))) uint16_t minCurrent = 0;
__attribute__((section(".appdata"))) uint16_t minCount = 0;
__attribute__((section(".appdata"))) uint16_t time = 0;
__attribute__((section(".appdata"))) unsigned int count = 0;
__attribute__((section(".appdata"))) unsigned int nradcconv = NRADCCONVMAX;

void main(void)
{
    WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
    __enable_interrupt();

    GPIO_Init();

    Ref_A_setReferenceVoltage(REF_A_BASE, REF_A_VREF1_2V);
    Ref_A_enableReferenceVoltage(REF_A_BASE);

    CS_Init();
    UART_Init();
    DMA_Init();
    ADC_Init();
    TIMER_A0_Init();
    TIMER_A1_Init();
    TIMER_B0_Init();

#ifdef MANUAL
    ADC12_B_startConversion(ADC12_B_BASE, ADC12_B_START_AT_ADC12MEM0, ADC12_B_REPEATED_SINGLECHANNEL);
    Timer_A_startCounter(TIMER_A0_BASE,TIMER_A_UP_MODE);
#endif

#ifdef ENABLE
    enableDRV();
    __delay_cycles(1000);
    disableDRV();
#endif

#ifdef TEST
    enableDRV();
    __delay_cycles(1000);
    disableDRV();
    __delay_cycles(500);
    enableDRV();
    __delay_cycles(1000);
    disableDRV();
    __delay_cycles(500);
    enableDRV();
    __delay_cycles(1000);
    disableDRV();
    __delay_cycles(500);
    enableDRV();
    __delay_cycles(1000);
    disableDRV();
#endif

#ifdef HOLD
    enableDRV();
    __delay_cycles(200);
    calcAvrg();
    calcParam();

    sendAvrg();
#endif
            while(1)
            {
           __bis_SR_register(LPM0_bits + GIE);
           __no_operation();
            }
}



