/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include "peripherals.h"

__attribute__((section(".peripheraldata"))) ADC12_B_initParam initParamADC;
__attribute__((section(".peripheraldata"))) ADC12_B_configureMemoryParam configParamADC;
__attribute__((section(".peripheraldata"))) DMA_initParam initParamDMA_Ch0;
__attribute__((section(".peripheraldata"))) EUSCI_A_UART_initParam initParamUART;
__attribute__((section(".peripheraldata"))) Timer_A_initUpModeParam initUpParamTimerA;
__attribute__((section(".peripheraldata"))) Timer_A_initCompareModeParam initCompareParamTimerA;
__attribute__((section(".peripheraldata"))) Timer_A_initUpModeParam initUpParamTimerA1;
__attribute__((section(".peripheraldata"))) Timer_B_initUpModeParam initUpParamTimerB;
__attribute__((section(".peripheraldata"))) Timer_B_initCompareModeParam initCompareParamTimerB;

bool mode = 1;
extern uint16_t actualData[NRADCCONVMAX];
extern unsigned int nradcconv;
uint8_t dutyCycle = 55;

void GPIO_Init(void)
{
	GPIO_setAsOutputPin(GPIO_PORT_P1,GPIO_PIN0|GPIO_PIN2|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P2,GPIO_PIN2|GPIO_PIN3|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P3,GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P4,GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN4|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_PJ,GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);

	GPIO_setAsInputPin(GPIO_PORT_P2, GPIO_PIN5|GPIO_PIN6);
	GPIO_setAsInputPin(GPIO_PORT_P1, GPIO_PIN3|GPIO_PIN4);
	GPIO_setAsInputPin(GPIO_PORT_P4, GPIO_PIN3);

	GPIO_setAsInputPinWithPullDownResistor(GPIO_PORT_P1,GPIO_PIN1);
	GPIO_setAsInputPinWithPullDownResistor(GPIO_PORT_P4,GPIO_PIN5);
	GPIO_selectInterruptEdge(GPIO_PORT_P1,GPIO_PIN1,GPIO_LOW_TO_HIGH_TRANSITION);
	GPIO_selectInterruptEdge(GPIO_PORT_P4,GPIO_PIN5,GPIO_LOW_TO_HIGH_TRANSITION);

	GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_PJ, GPIO_PIN4 + GPIO_PIN5, GPIO_PRIMARY_MODULE_FUNCTION);					// LFXTOUT and LFXTIN
	GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P4,GPIO_PIN2,GPIO_TERNARY_MODULE_FUNCTION);								// A10
	GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P2, GPIO_PIN0 + GPIO_PIN1, GPIO_SECONDARY_MODULE_FUNCTION);				// P2.0 and P2.1 UCA0TXD und UCA0RXD

	GPIO_setOutputLowOnPin(GPIO_PORT_P1,GPIO_PIN0|GPIO_PIN2|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P2,GPIO_PIN2|GPIO_PIN3|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P3,GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P4,GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN4|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_PJ,GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);

	PMM_unlockLPM5();
	__enable_interrupt();

	GPIO_clearInterrupt(GPIO_PORT_P1,GPIO_PIN1);
	GPIO_clearInterrupt(GPIO_PORT_P4,GPIO_PIN5);
	GPIO_enableInterrupt(GPIO_PORT_P1,GPIO_PIN1);
	GPIO_enableInterrupt(GPIO_PORT_P4,GPIO_PIN5);
}

void CS_Init(void)
{
	//GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_PJ, GPIO_PIN4 + GPIO_PIN5, GPIO_PRIMARY_MODULE_FUNCTION);					// LFXTOUT and LFXTIN

    //Set DCO frequency to 16 MHz
    CS_setDCOFreq(CS_DCORSEL_1,CS_DCOFSEL_4);											// High Frequency Option, 16MHz
    //Set external clock frequency to 32.768 KHz
    //CS_setExternalClockSource(32768,0);
    //Set ACLK=LFXT
    //CS_initClockSignal(CS_ACLK,CS_LFXTCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Set SMCLK = LFXT with frequency divider of 1
    CS_initClockSignal(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_2);					// 8 MHz max speed for FRAM access
    //Set MCLK = LFXT with frequency divider of 1
    //ToDo: change to DCOCLK; did not work before
    //CS_initClockSignal(CS_MCLK,CS_LFXTCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Start XT1 with no time out
    //CS_turnOnLFXT(CS_LFXT_DRIVE_3);
    //Set wait state to 1
    FRAMCtl_configureWaitStateControl(FRAMCTL_ACCESS_TIME_CYCLES_1);
}

void ADC_Init(void)
{
	initParamADC.sampleHoldSignalSourceSelect = ADC12_B_SAMPLEHOLDSOURCE_1;				// TA0 CCR1 output
	initParamADC.clockSourceSelect = ADC12_B_CLOCKSOURCE_ADC12OSC;						// 5MHz MODOSC
	initParamADC.clockSourcePredivider = ADC12_B_CLOCKPREDIVIDER__1;
	initParamADC.clockSourceDivider = ADC12_B_CLOCKDIVIDER_1;
	initParamADC.internalChannelMap = ADC12_B_NOINTCH;

	configParamADC.differentialModeSelect = ADC12_B_DIFFERENTIAL_MODE_DISABLE;
	configParamADC.endOfSequence = ADC12_B_NOTENDOFSEQUENCE;
	configParamADC.inputSourceSelect = ADC12_B_INPUT_A10;
	configParamADC.memoryBufferControlIndex = ADC12_B_MEMORY_0;
	configParamADC.refVoltageSourceSelect = ADC12_B_VREFPOS_INTBUF_VREFNEG_VSS;
	configParamADC.windowComparatorSelect = ADC12_B_WINDOW_COMPARATOR_DISABLE;

	ADC12_B_init(ADC12_B_BASE, &initParamADC);

	ADC12_B_setupSamplingTimer(ADC12_B_BASE,
							   ADC12_B_CYCLEHOLD_64_CYCLES,
							   ADC12_B_CYCLEHOLD_64_CYCLES,
							   ADC12_B_MULTIPLESAMPLESDISABLE);

	ADC12_B_configureMemory(ADC12_B_BASE, &configParamADC);

	ADC12_B_enable(ADC12_B_BASE);
}

void UART_Init(void)
{
	initParamUART.selectClockSource = EUSCI_A_UART_CLOCKSOURCE_SMCLK;
	initParamUART.clockPrescalar = 52;
	initParamUART.firstModReg = 1;
	initParamUART.secondModReg = 0x49;
	initParamUART.parity = EUSCI_A_UART_NO_PARITY;
	initParamUART.msborLsbFirst = EUSCI_A_UART_LSB_FIRST;
	initParamUART.numberofStopBits = EUSCI_A_UART_ONE_STOP_BIT;
	initParamUART.uartMode = EUSCI_A_UART_MODE;
	initParamUART.overSampling = EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION;

	EUSCI_A_UART_init(EUSCI_A0_BASE,&initParamUART);
	EUSCI_A_UART_enable(EUSCI_A0_BASE);

	EUSCI_A_UART_clearInterrupt(EUSCI_A0_BASE, EUSCI_A_UART_RECEIVE_INTERRUPT_FLAG);
	EUSCI_A_UART_enableInterrupt(EUSCI_A0_BASE, EUSCI_A_UART_RECEIVE_INTERRUPT);
}

void UART_send(uint16_t data)
{
	while(!(UCA0IFG&UCTXIFG));
	UCA0TXBUF=HI(data);
	while(!(UCA0IFG&UCTXIFG));
	UCA0TXBUF=LO(data);
	while(!(UCA0IFG&UCTXIFG));
	UCA0TXBUF='\r';
	while(!(UCA0IFG&UCTXIFG));
	UCA0TXBUF='\n';
}

void DMA_Init(void)
{
	initParamDMA_Ch0.channelSelect = DMA_CHANNEL_0;
	initParamDMA_Ch0.transferModeSelect = DMA_TRANSFER_SINGLE;
	initParamDMA_Ch0.transferSize = nradcconv;
	initParamDMA_Ch0.triggerSourceSelect = DMA0TSEL__ADC12IFG;									// triggered by ADC
	initParamDMA_Ch0.transferUnitSelect = DMA_SIZE_SRCWORD_DSTWORD;
	initParamDMA_Ch0.triggerTypeSelect = DMA_TRIGGER_RISINGEDGE;

	DMA_init(&initParamDMA_Ch0);
	DMA_setSrcAddress(DMA_CHANNEL_0,ADC12MEM0,DMA_DIRECTION_UNCHANGED);
	DMA_setDstAddress(DMA_CHANNEL_0,(uint32_t)&actualData[0],DMA_DIRECTION_INCREMENT);
	DMA_enableInterrupt(DMA_CHANNEL_0);
	DMA_enableTransfers(DMA_CHANNEL_0);
}

void TIMER_A0_Init(void)
{
	initUpParamTimerA.clockSource = TIMER_A_CLOCKSOURCE_SMCLK;													// 8MHz
	initUpParamTimerA.clockSourceDivider = TIMER_A_CLOCKSOURCE_DIVIDER_1;
	initUpParamTimerA.timerPeriod = TIMER_A_PERIOD;
	initUpParamTimerA.timerInterruptEnable_TAIE = TIMER_A_TAIE_INTERRUPT_DISABLE;
	initUpParamTimerA.captureCompareInterruptEnable_CCR0_CCIE = TIMER_A_CCIE_CCR0_INTERRUPT_DISABLE;
	initUpParamTimerA.timerClear = TIMER_A_SKIP_CLEAR;
	initUpParamTimerA.startTimer = false;

	Timer_A_initUpMode(TIMER_A0_BASE,&initUpParamTimerA);

	initCompareParamTimerA.compareRegister = TIMER_A_CAPTURECOMPARE_REGISTER_1;
	initCompareParamTimerA.compareInterruptEnable = TIMER_A_CAPTURECOMPARE_INTERRUPT_DISABLE;
	initCompareParamTimerA.compareOutputMode = TIMER_A_OUTPUTMODE_SET_RESET;
	initCompareParamTimerA.compareValue = TIMER_A_DUTY_CYCLE;

	Timer_A_initCompareMode(TIMER_A0_BASE, &initCompareParamTimerA);
}

void TIMER_A1_Init(void)
{
	initUpParamTimerA1.clockSource = TIMER_A_CLOCKSOURCE_SMCLK;													// 8MHz
	initUpParamTimerA1.clockSourceDivider = TIMER_A_CLOCKSOURCE_DIVIDER_64;										// 125kHz Clock
	initUpParamTimerA1.timerPeriod = TIMER_A1_PERIOD;															// 33,3Hz
	initUpParamTimerA1.timerInterruptEnable_TAIE = TIMER_A_TAIE_INTERRUPT_DISABLE;
	initUpParamTimerA1.captureCompareInterruptEnable_CCR0_CCIE = TIMER_A_CCIE_CCR0_INTERRUPT_ENABLE;
	initUpParamTimerA1.timerClear = TIMER_A_SKIP_CLEAR;
	initUpParamTimerA1.startTimer = false;

	Timer_A_initUpMode(TIMER_A1_BASE,&initUpParamTimerA1);
	Timer_A_clearCaptureCompareInterrupt(TIMER_A1_BASE, TIMER_A_CAPTURECOMPARE_REGISTER_0);
}

void TIMER_B0_Init(void)
{
	    initUpParamTimerB.clockSource = TIMER_B_CLOCKSOURCE_SMCLK;
	    initUpParamTimerB.clockSourceDivider = TIMER_B_CLOCKSOURCE_DIVIDER_1;
	    initUpParamTimerB.timerPeriod = TIMER_B_PERIOD;
	    initUpParamTimerB.timerInterruptEnable_TBIE = TIMER_B_TBIE_INTERRUPT_DISABLE;
	    initUpParamTimerB.captureCompareInterruptEnable_CCR0_CCIE = TIMER_B_CCIE_CCR0_INTERRUPT_DISABLE;
	    initUpParamTimerB.timerClear = TIMER_B_DO_CLEAR;//DO_CLEAR
	    initUpParamTimerB.startTimer = true;

	    initCompareParamTimerB.compareRegister = TIMER_B_CAPTURECOMPARE_REGISTER_5;
	    initCompareParamTimerB.compareInterruptEnable = TIMER_B_CAPTURECOMPARE_INTERRUPT_DISABLE;
	    initCompareParamTimerB.compareOutputMode = TIMER_B_OUTPUTMODE_RESET_SET;

	    Timer_B_clearCaptureCompareInterrupt(TIMER_B0_BASE, TIMER_B_CAPTURECOMPARE_REGISTER_5);
}

#pragma vector=ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{
	switch(__even_in_range(ADC12IV,12))
	{
	case 0: break;
	case 2: break;
	case 4: break;
	case 6: break;
	case 8: break;
	case 10: break;
	case 12: break;
	case 14: break;
	case 16: break;
	case 18: break;
	case 20: break;
	case 22: break;
	case 24: break;
	case 26: break;
	case 28: break;
	case 30: break;
	case 32: break;
	case 34: break;
	case 36: break;
	case 38: break;
	case 40: break;
	case 42: break;
	case 44: break;
	case 46: break;
	case 48: break;
	case 50: break;
	case 52: break;
	case 54: break;
	case 56: break;
	case 58: break;
	case 60: break;
	case 62: break;
	case 64: break;
	case 66: break;
	case 68: break;
	case 70: break;
	case 72: break;
	case 74: break;
	case 76: break;
	}
}

#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	static uint8_t command1 = 0;
	static uint8_t command2 = 0;
	static uint8_t command3 = 0;
	bool on = false;
	static uint8_t count=0;

	switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
	{
		case USCI_NONE:	break;
		case USCI_UART_UCRXIFG:																// receive buffer interrupt
			if(count ==0)
			{
				command1 = EUSCI_A_UART_receiveData(EUSCI_A0_BASE);
			}
			else if(count ==1)
			{
				command2 = EUSCI_A_UART_receiveData(EUSCI_A0_BASE);
			}
			else if(count ==2)
			{
				command3 = EUSCI_A_UART_receiveData(EUSCI_A0_BASE);
			}
			count++;

			if(count ==3)
			{
				switch(command1)	// on/off command
				{
					case ON:
						on = true;
						break;
					case OFF:
						on= false;
						break;
					default: break;
				}
				if(on)
				{
					switch(command2)	// duty cycle
					{
					case DUTY_CYCLE_10:
						dutyCycle = 5;
						break;
					case DUTY_CYCLE_20:
						dutyCycle = 10;
						break;
					case DUTY_CYCLE_30:
						dutyCycle = 15;
						break;
					case DUTY_CYCLE_40:
						dutyCycle = 20;
						break;
					case DUTY_CYCLE_50:
						dutyCycle = 25;
						break;
					case DUTY_CYCLE_60:
						dutyCycle = 30;
						break;
					case DUTY_CYCLE_70:
						dutyCycle = 35;
						break;
					case DUTY_CYCLE_80:
						dutyCycle = 40;
						break;
					case DUTY_CYCLE_90:
						dutyCycle = 45;
						break;
					case DUTY_CYCLE_100:
						dutyCycle = 50;
						break;
					default: break;
					}

					switch(command3)	//# of a/d conversions
					{
					case NRADCCONV_100:
						nradcconv = 100;
						break;
					case NRADCCONV_250:
						nradcconv = 250;
						break;
					case NRADCCONV_500:
						nradcconv = 500;
						break;
					default: break;
					}
					enableDRV();
				}
				else if (!on)
				{
					switch(command2)	// data to send to host PC
					{
					case DATA_RAW:
						sendRaw();
						break;
					case DATA_AVRG:
						calcAvrg();
						sendAvrg();
						break;
					case DATA_MIN_MAX:
						calcAvrg();
						calcParam();
						sendMinMax();
						break;
					case DATA_ALL:
						calcAvrg();
						calcParam();
						sendRaw();
						sendAvrg();
						sendMinMax();
						break;
					default: break;
					}
					disableDRV();
				}
				count = 0;
			}

			break;
		case USCI_UART_UCTXIFG: break;
		case USCI_UART_UCSTTIFG: break;
		case USCI_UART_UCTXCPTIFG: break;
	}
}

#pragma vector=DMA_VECTOR
__interrupt void DMA_ISR(void)
{
	switch(__even_in_range(DMAIV,DMAIV_DMA2IFG))
	{
		case DMAIV_NONE: break;
		case DMAIV_DMA0IFG:
			Timer_A_stop(TIMER_A0_BASE);
			ADC12_B_disableConversions(ADC12_B_BASE,ADC12_B_COMPLETECONVERSION);

			DMA_clearInterrupt(DMA_CHANNEL_0);
			DMA_enableTransfers(DMA_CHANNEL_0);

			break;
		case DMAIV_DMA1IFG:	break;
		case DMAIV_DMA2IFG: break;
	}
}

#pragma vector = TIMER0_A1_VECTOR
__interrupt void TIMER0_A1_ISR(void)
{
  	switch(__even_in_range(TA0IV,TA0IV_TAIFG))
	{
	case TA0IV_NONE: break;
	case TA0IV_TACCR1: break;
	case TA0IV_TACCR2: break;
	case TA0IV_3: break;
	case TA0IV_4: break;
	case TA0IV_5: break;
	case TA0IV_6: break;
	case TA0IV_TAIFG: break;
	}
}
#pragma vector = TIMER1_A0_VECTOR
__interrupt void TIMER1_A0_ISR(void)
{

	Timer_B_initUpMode(TIMER_B0_BASE, &initUpParamTimerB);
	initCompareParamTimerB.compareValue = dutyCycle;
	Timer_B_initCompareMode(TIMER_B0_BASE, &initCompareParamTimerB);

	GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P3, GPIO_PIN6, GPIO_PRIMARY_MODULE_FUNCTION);
	PMM_unlockLPM5();

	Timer_A_stop(TIMER_A1_BASE);
}

#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
{
	switch(__even_in_range(P1IV,P1IV_P1IFG7))
	{
	case P1IV_P1IFG0:break;
	case P1IV_P1IFG1:
		dutyCycle = 55;
	    break;
	case P1IV_P1IFG2:break;
	case P1IV_P1IFG3:break;
	case P1IV_P1IFG4:break;
	case P1IV_P1IFG5:break;
	case P1IV_P1IFG6:break;
	case P1IV_P1IFG7:break;
	default:
		__no_operation();
		_never_executed();
	}
}

#pragma vector=PORT4_VECTOR
__interrupt void Port_4(void)
{
	static bool on=false;

	switch(__even_in_range(P4IV,P4IV_P4IFG7))
	{
	case P4IV_P4IFG0:break;
	case P4IV_P4IFG1:break;
	case P4IV_P4IFG2:break;
	case P4IV_P4IFG3:break;
	case P4IV_P4IFG4:break;
	case P4IV_P4IFG5:
		GPIO_setOutputLowOnPin(GPIO_PORT_P3,GPIO_PIN2);
		if(!on)
		{
			dutyCycle = dutyCycle - 5;
			enableDRV();

			if(dutyCycle == 0)
			{
				dutyCycle = 55;
				GPIO_setOutputLowOnPin(GPIO_PORT_P3,GPIO_PIN2);					// blue LED off

			}
			else if(dutyCycle == 5)
			{
				GPIO_setOutputHighOnPin(GPIO_PORT_P3,GPIO_PIN2);				// blue LED on
			}
			on = true;
		}
		else if (on)
		{
			disableDRV();
			calcAvrg();
			sendAvrg();
			on = false;
		}
    break;
	case P4IV_P4IFG6:break;
	case P4IV_P4IFG7:break;
	default:
		__no_operation();
		_never_executed();
	}
}
