/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include "signal.h"

extern uint16_t minCurrent;
extern uint16_t maxCurrent;
extern uint16_t time;
extern uint16_t minCount;
extern uint16_t maxCount;
uint8_t switchCount = 0;

extern uint16_t avrgData[NRADCCONVMAX];
extern uint16_t actualData[NRADCCONVMAX];
extern uint16_t saveData[NRADCCONVMAX];
extern unsigned int nradcconv;

void UART_send(uint16_t data);

void calcAvrg(void)
{
	uint16_t counter = 0;

	while(counter<nradcconv)
	{
#ifdef AVRG4												// moving average always div=4

			if(counter == 0)
			{
				avrgData[counter] = actualData[counter];
				counter++;
			}
			else if(counter == 1)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1])/2;
				counter++;
			}
			else if (counter == 2)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2])/3;
				counter++;
			}
			else if(counter >= 3)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2]+actualData[counter-3])/4;
				counter++;
			}
#endif
#ifdef AVRG8												// moving average always div=8
			if(counter == 0)
			{
				avrgData[counter] = actualData[counter];
			}
			else if(counter == 1)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1])/2;
			}
			else if (counter == 2)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2])/3;
			}
			else if(counter == 3)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2]+actualData[counter-3])/4;
			}
			else if(counter == 4)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2]+actualData[counter-3]+actualData[counter-4])/5;
			}
			else if(counter == 5)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2]+actualData[counter-3]+actualData[counter-4]+actualData[counter-5])/6;
			}
			else if(counter == 6)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2]+actualData[counter-3]+actualData[counter-4]+actualData[counter-5]+actualData[counter-6])/7;
			}
			else if(counter >= 7)
			{
				avrgData[counter]= (actualData[counter]+actualData[counter-1]+actualData[counter-2]+actualData[counter-3]+actualData[counter-4]+actualData[counter-5]+actualData[counter-6]+actualData[counter-7])/8;
			}
			counter++;
#endif

	}
	counter = 0;
}

void calcMin(void)
{
	unsigned int i = maxCount;
	uint16_t min1, min2 = 0;
	for(; i<nradcconv;i++)
	{
		if(avrgData[i] <= avrgData[i-1] && avrgData[i] <= avrgData[i-2] &&  avrgData[i] <= avrgData[i-3] && avrgData[i] <= avrgData[i+1] && avrgData[i] <= avrgData[i+2] &&  avrgData[i] <= avrgData[i+3])
		{
			min1 = avrgData[i];
			break;
		}
	}
	minCurrent = min1;
	minCount = i;

	while(i<nradcconv)
	{
		for(; i < nradcconv; i++)
		{
			if(avrgData[i] <= avrgData[i-1] && avrgData[i] <= avrgData[i-2] &&  avrgData[i] <= avrgData[i-3] && avrgData[i] <= avrgData[i+1] && avrgData[i] <= avrgData[i+2] &&  avrgData[i] <= avrgData[i+3])
			{
				min2 = avrgData[i];
			}
		}

		if(min2 < minCurrent)
		{
			minCurrent = min2;
			minCount = i;
		}
	}
	saveData[3*switchCount+1] = minCurrent;
}

void calcMax(void)
{
	unsigned int i = 0;
	for(; i<nradcconv; i++)
	{
		if(i>2)
		{
			if(avrgData[i] >= 200 && avrgData[i] >= avrgData[i-1] && avrgData[i] >= avrgData[i-2] && avrgData[i] >= avrgData[i-3] && avrgData[i] >= avrgData[i+1] && avrgData[i] >= avrgData[i+2] && avrgData[i] >= avrgData[i+3])
			{
				maxCurrent = avrgData[i];
				maxCount = i;
				break;
			}
		}
		else if(i>0 && i<=2)
		{
			maxCurrent = avrgData[i];
		}
	}
	saveData[3*switchCount+0] = maxCurrent;
}

void calcTime(void)
{
	uint8_t deltaCount = 0;

	deltaCount =  minCount - maxCount;

	time = deltaCount * 128;					// in us;               8MHz clock, 1024 timer period; 7812,5 sampling frequency
	saveData[3*switchCount+2] = time;
}

void calcParam(void)
{
	calcMax();
	calcMin();
	calcTime();

	if(switchCount >=nradcconv)
	{
		switchCount = 0;
	}
	else
		switchCount++;
}

void sendMinMax(void)
{
	UART_send(maxCurrent);
	UART_send(minCurrent);
	UART_send(time);
}

void sendAvrg(void)
{
	int i;

	for(i=0;i<nradcconv;i++)
	{
		UART_send(avrgData[i]);
	}
}

void sendRaw(void)
{
	int i;

		for(i=0;i<nradcconv;i++)
		{
			UART_send(actualData[i]);
		}
}
